/* How to set graphics mode, palette and use keyboard with SvgaLib.

Based on the SvgaLib example: keytest.

Distribution under GNU-GPL.
Autor : Leandro Bernsmuller 
e-mail: lebe@geocities.com
site  : http://www.geocities.com/SiliconValley/Bay/1704

To compile just type make.

usualy headers are /usr/include/vga*.h
*/



#include <string.h>       // memset
#include <stdio.h>        // getchar
#include <vga.h>          // vga lib functions
#include <vgakeyboard.h>  // vga keyboard functions



int main ()
{
   // init SvgaLib
   vga_init ();
  
   printf ( "\nGraphics under Linux console with SvgaLib." );
   printf ( "\n\nDemo 3 - using keyboard with no typematic rates." );
   printf ( "\n\nTry press cursor arrow keys (UP, DOWN, RIGHT, LEFT) to move point (has diagonal).\nESC exit." );
   printf ( "\n\nPress ENTER..." );    
   getchar ();
 
   // set graphics mode and palette
   vga_setmode ( G320x200x256 );
   vga_setpalette ( 0, 0, 0, 30 );
   vga_setpalette ( 1, 50, 50, 50 );

   // try set keyboard to RAW mode
   printf ( "Trying to init keyboard.\n" );
   if ( keyboard_init () ) {
      printf ( "Could not initialize keyboard.\n" );
      exit ( 1 );
   }
   // set keypad and cursor keys as having same action, enable diagonal
   keyboard_translatekeys ( TRANSLATE_CURSORKEYS | TRANSLATE_KEYPADENTER |
                            TRANSLATE_DIAGONAL );
   
   // where to draw
   unsigned char
      auchFrameBuffer [ 64000 ];
   
   // clear framebuffer
   memset ( (void*) auchFrameBuffer, 0, 64000 );

   int
      nX = 160,   // pixel coordenated
      nY = 100;
   // loop
   for ( ; ; ) {
 
      // read keyboard status
      keyboard_update();
 
      // change coordenates acording the pressed keys
      if ( keyboard_keypressed ( SCANCODE_CURSORLEFT ) )
         nX = ( nX <= 0 ) ? nX : nX - 1;
      if ( keyboard_keypressed ( SCANCODE_CURSORRIGHT ) )
         nX = ( nX >= 319 ) ? nX : nX + 1;
      if ( keyboard_keypressed ( SCANCODE_CURSORUP ) )
         nY = ( nY <= 0 ) ? nY : nY - 1;
      if ( keyboard_keypressed ( SCANCODE_CURSORDOWN ) )
        nY = ( nY >= 199 ) ? nY : nY + 1;

      // test if ESC was pressed -> exit
      if ( keyboard_keypressed ( SCANCODE_ESCAPE ) )
         break;

      // set a pixel on frame buffer
      memset ( (void*) ( auchFrameBuffer + 320 * nY + nX ), 1, 1 );
     
      // wait vertical retrace to update frame buffer
      vga_waitretrace ();
 
      // fade palette
      static int
         nComp = 0,
         nInc = 1;
      nComp += nInc;
      if ( nComp == 63 || nComp == 0 )
         nInc *= -1;
      vga_setpalette ( 0, 30, 30, nComp );
 
      // show frame buffer
      // draw scanline per scanline on screen
      for ( int ncL = 0; ncL < 200; ncL ++ )
         vga_drawscansegment ( auchFrameBuffer + ncL * 320, 0, ncL, 320 );
   }
   
   // end keyboard
   keyboard_close ();
 
   vga_setmode ( TEXT );
}

